@@ -1,8 +1,8 @@
 # --
 # Kernel/System/Auth/LDAP.pm - provides the ldap authentification
-# Copyright (C) 2001-2006 OTRS GmbH, http://otrs.org/
+# Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: LDAP.pm,v 1.28 2006/11/08 15:37:03 cs Exp $
+# $Id: LDAP.pm,v 1.38 2007/07/26 12:59:26 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -12,11 +12,12 @@
 package Kernel::System::Auth::LDAP;
 
 use strict;
+use warnings;
 use Net::LDAP;
 use Kernel::System::Encode;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.28 $';
+$VERSION = '$Revision: 1.38 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 sub new {
@@ -39,26 +40,52 @@ sub new {
     $Self->{Debug} = 0;
 
     # get ldap preferences
-    $Self->{Host} = $Self->{ConfigObject}->Get('AuthModule::LDAP::Host')
-     || die "Need AuthModule::LDAP::Host in Kernel/Config.pm";
-    $Self->{BaseDN} = $Self->{ConfigObject}->Get('AuthModule::LDAP::BaseDN')
-     || die "Need AuthModule::LDAP::BaseDN in Kernel/Config.pm";
-    $Self->{UID} = $Self->{ConfigObject}->Get('AuthModule::LDAP::UID')
-     || die "Need AuthModule::LDAP::UID in Kernel/Config.pm";
-    $Self->{SearchUserDN} = $Self->{ConfigObject}->Get('AuthModule::LDAP::SearchUserDN') || '';
-    $Self->{SearchUserPw} = $Self->{ConfigObject}->Get('AuthModule::LDAP::SearchUserPw') || '';
-    $Self->{GroupDN} = $Self->{ConfigObject}->Get('AuthModule::LDAP::GroupDN') || '';
-    $Self->{AccessAttr} = $Self->{ConfigObject}->Get('AuthModule::LDAP::AccessAttr') || 'memberUid';
-    $Self->{UserAttr} = $Self->{ConfigObject}->Get('AuthModule::LDAP::UserAttr') || 'DN';
-    $Self->{UserSuffix} = $Self->{ConfigObject}->Get('AuthModule::LDAP::UserSuffix') || '';
-    $Self->{UserLowerCase} = $Self->{ConfigObject}->Get('AuthModule::LDAP::UserLowerCase') || 0;
-    $Self->{DestCharset} = $Self->{ConfigObject}->Get('AuthModule::LDAP::Charset') || 'utf-8';
+    $Self->{Count} = $Param{Count} || '';
+    $Self->{Die} = $Self->{ConfigObject}->Get('AuthModule::LDAP::Die'.$Param{Count});
+    if ($Self->{ConfigObject}->Get('AuthModule::LDAP::Host'.$Param{Count})) {
+        $Self->{Host} = $Self->{ConfigObject}->Get('AuthModule::LDAP::Host'.$Param{Count});
+    }
+    else {
+        $Self->{LogObject}->Log(
+            Priority => 'error',
+            Message => "Need AuthModule::LDAP::Host$Param{Count} in Kernel/Config.pm",
+        );
+        return;
+    }
+    if (defined($Self->{ConfigObject}->Get('AuthModule::LDAP::BaseDN'.$Param{Count}))) {
+        $Self->{BaseDN} = $Self->{ConfigObject}->Get('AuthModule::LDAP::BaseDN'.$Param{Count});
+    }
+    else {
+        $Self->{LogObject}->Log(
+            Priority => 'error',
+            Message => "Need AuthModule::LDAP::BaseDN$Param{Count} in Kernel/Config.pm",
+        );
+        return;
+    }
+    if ($Self->{ConfigObject}->Get('AuthModule::LDAP::UID'.$Param{Count})) {
+        $Self->{UID} = $Self->{ConfigObject}->Get('AuthModule::LDAP::UID'.$Param{Count});
+    }
+    else {
+        $Self->{LogObject}->Log(
+            Priority => 'error',
+            Message => "Need AuthModule::LDAP::UID$Param{Count} in Kernel/Config.pm",
+        );
+        return;
+    }
+    $Self->{SearchUserDN} = $Self->{ConfigObject}->Get('AuthModule::LDAP::SearchUserDN'.$Param{Count}) || '';
+    $Self->{SearchUserPw} = $Self->{ConfigObject}->Get('AuthModule::LDAP::SearchUserPw'.$Param{Count}) || '';
+    $Self->{GroupDN} = $Self->{ConfigObject}->Get('AuthModule::LDAP::GroupDN'.$Param{Count}) || '';
+    $Self->{AccessAttr} = $Self->{ConfigObject}->Get('AuthModule::LDAP::AccessAttr'.$Param{Count}) || 'memberUid';
+    $Self->{UserAttr} = $Self->{ConfigObject}->Get('AuthModule::LDAP::UserAttr'.$Param{Count}) || 'DN';
+    $Self->{UserSuffix} = $Self->{ConfigObject}->Get('AuthModule::LDAP::UserSuffix'.$Param{Count}) || '';
+    $Self->{UserLowerCase} = $Self->{ConfigObject}->Get('AuthModule::LDAP::UserLowerCase'.$Param{Count}) || 0;
+    $Self->{DestCharset} = $Self->{ConfigObject}->Get('AuthModule::LDAP::Charset'.$Param{Count}) || 'utf-8';
 
     # ldap filter always used
-    $Self->{AlwaysFilter} = $Self->{ConfigObject}->Get('AuthModule::LDAP::AlwaysFilter') || '';
+    $Self->{AlwaysFilter} = $Self->{ConfigObject}->Get('AuthModule::LDAP::AlwaysFilter'.$Param{Count}) || '';
     # Net::LDAP new params
-    if ($Self->{ConfigObject}->Get('AuthModule::LDAP::Params')) {
-        $Self->{Params} = $Self->{ConfigObject}->Get('AuthModule::LDAP::Params');
+    if ($Self->{ConfigObject}->Get('AuthModule::LDAP::Params'.$Param{Count})) {
+        $Self->{Params} = $Self->{ConfigObject}->Get('AuthModule::LDAP::Params'.$Param{Count});
     }
     else {
         $Self->{Params} = {};
@@ -120,13 +147,26 @@ sub Auth {
     # just in case for debug!
     if ($Self->{Debug} > 0) {
         $Self->{LogObject}->Log(
-          Priority => 'notice',
-          Message => "User: '$Param{User}' tried to authenticate with Pw: '$Param{Pw}' (REMOTE_ADDR: $RemoteAddr)",
+            Priority => 'notice',
+            Message => "User: '$Param{User}' tried to authenticate with Pw: '$Param{Pw}' ".
+                "(REMOTE_ADDR: $RemoteAddr)",
         );
     }
 
     # ldap connect and bind (maybe with SearchUserDN and SearchUserPw)
-    my $LDAP = Net::LDAP->new($Self->{Host}, %{$Self->{Params}}) or die "$@";
+    my $LDAP = Net::LDAP->new($Self->{Host}, %{$Self->{Params}});
+    if (!$LDAP) {
+        if ($Self->{Die}) {
+            die "Can't connect to $Self->{Host}: $@";
+        }
+        else {
+            $Self->{LogObject}->Log(
+                Priority => 'error',
+                Message => "Can't connect to $Self->{Host}: $@",
+            );
+            return;
+        }
+    }
     my $Result = '';
     if ($Self->{SearchUserDN} && $Self->{SearchUserPw}) {
         $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
@@ -149,7 +189,7 @@ sub Auth {
     }
     # perform user search
     $Result = $LDAP->search (
-        base   => $Self->{BaseDN},
+        base => $Self->{BaseDN},
         filter => $Filter,
     );
     if ($Result->code) {
@@ -168,9 +208,9 @@ sub Auth {
     if (!$UserDN) {
         # failed login note
         $Self->{LogObject}->Log(
-          Priority => 'notice',
-          Message => "User: $Param{User} authentication failed, no LDAP entry found!".
-            "BaseDN='$Self->{BaseDN}', Filter='$Filter', (REMOTE_ADDR: $RemoteAddr).",
+            Priority => 'notice',
+            Message => "User: $Param{User} authentication failed, no LDAP entry found!".
+                "BaseDN='$Self->{BaseDN}', Filter='$Filter', (REMOTE_ADDR: $RemoteAddr).",
         );
         # take down session
         $LDAP->unbind;
@@ -195,7 +235,7 @@ sub Auth {
             $Filter2 = "($Self->{AccessAttr}=$Param{User})";
         }
         my $Result2 = $LDAP->search (
-            base   => $Self->{GroupDN},
+            base => $Self->{GroupDN},
             filter => $Filter2,
         );
         if ($Result2->code) {
@@ -214,9 +254,9 @@ sub Auth {
         if (!$GroupDN) {
             # failed login note
             $Self->{LogObject}->Log(
-              Priority => 'notice',
-              Message => "User: $Param{User} authentication failed, no LDAP group entry found".
-                "GroupDN='$Self->{GroupDN}', Filter='$Filter2'! (REMOTE_ADDR: $RemoteAddr).",
+                Priority => 'notice',
+                Message => "User: $Param{User} authentication failed, no LDAP group entry found".
+                    "GroupDN='$Self->{GroupDN}', Filter='$Filter2'! (REMOTE_ADDR: $RemoteAddr).",
             );
             # take down session
             $LDAP->unbind;
@@ -229,8 +269,8 @@ sub Auth {
     if ($Result->code) {
         # failed login note
         $Self->{LogObject}->Log(
-          Priority => 'notice',
-          Message => "User: $Param{User} ($UserDN) authentication failed: '".$Result->error()."' (REMOTE_ADDR: $RemoteAddr).",
+            Priority => 'notice',
+            Message => "User: $Param{User} ($UserDN) authentication failed: '".$Result->error()."' (REMOTE_ADDR: $RemoteAddr).",
         );
         # take down session
         $LDAP->unbind;
@@ -251,12 +291,18 @@ sub Auth {
             Message => "User: $Param{User} ($UserDN) authentication ok (REMOTE_ADDR: $RemoteAddr).",
         );
         # sync user from ldap
-        if ($Self->{ConfigObject}->Get('UserSyncLDAPMap')) {
-            my $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+        if ($Self->{ConfigObject}->Get('UserSyncLDAPMap'.$Self->{Count})) {
+            my $Result = '';
+            if ($Self->{SearchUserDN} && $Self->{SearchUserPw}) {
+                $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+            }
+            else {
+                $Result = $LDAP->bind();
+            }
             if ($Result->code) {
                 $Self->{LogObject}->Log(
-                  Priority => 'error',
-                  Message => "Sync bind failed! ".$Result->error,
+                    Priority => 'error',
+                    Message => "Sync bind failed! ".$Result->error,
                 );
                 # take down session
                 $LDAP->unbind;
@@ -270,7 +316,7 @@ sub Auth {
             }
             # perform user search
             $Result = $LDAP->search (
-                base   => $Self->{BaseDN},
+                base => $Self->{BaseDN},
                 filter => $Filter,
             );
             if ($Result->code) {
@@ -284,10 +330,18 @@ sub Auth {
             my %SyncUser = ();
             foreach my $Entry ($Result->all_entries) {
                 $UserDN = $Entry->dn();
-                foreach (keys %{$Self->{ConfigObject}->Get('UserSyncLDAPMap')}) {
-                    $SyncUser{$_} = $Entry->get_value($Self->{ConfigObject}->Get('UserSyncLDAPMap')->{$_});
+                foreach my $Key (keys %{$Self->{ConfigObject}->Get('UserSyncLDAPMap'.$Self->{Count})}) {
+                    # detect old config setting
+                    if ($Key =~ /^(Firstname|Lastname|Email)/) {
+                        $Key = "User".$Key;
+                        $Self->{LogObject}->Log(
+                            Priority => 'error',
+                            Message => "Old config setting detected, please use the new one from Kernel/Config/Defaults.pm (User* has been added!).",
+                        );
+                    }
+                    $SyncUser{$Key} = $Entry->get_value($Self->{ConfigObject}->Get('UserSyncLDAPMap'.$Self->{Count})->{$Key});
                     # e. g. set utf-8 flag
-                    $SyncUser{$_} = $Self->_ConvertFrom($SyncUser{$_}, $Self->{ConfigObject}->Get('DefaultCharset'));
+                    $SyncUser{$Key} = $Self->_ConvertFrom($SyncUser{$Key}, $Self->{ConfigObject}->Get('DefaultCharset'));
                 }
                 if ($Entry->get_value('userPassword')) {
                     $SyncUser{Pw} = $Entry->get_value('userPassword');
@@ -301,22 +355,22 @@ sub Auth {
                 my %UserData = $Self->{UserObject}->GetUserData(User => $Param{User});
                 if (!%UserData) {
                     my $UserID = $Self->{UserObject}->UserAdd(
-                        Salutation => 'Mr/Mrs',
-                        Login => $Param{User},
+                        UserSalutation => 'Mr/Mrs',
+                        UserLogin => $Param{User},
                         %SyncUser,
                         UserType => 'User',
                         ValidID => 1,
-                        UserID => 1,
+                        ChangeUserID => 1,
                     );
                     if ($UserID) {
                         $Self->{LogObject}->Log(
                             Priority => 'notice',
                             Message => "Initial data for '$Param{User}' ($UserDN) created in RDBMS.",
                         );
-                        # sync inital groups
-                        if ($Self->{ConfigObject}->Get('UserSyncLDAPGroups')) {
+                        # sync initial groups
+                        if ($Self->{ConfigObject}->Get('UserSyncLDAPGroups'.$Self->{Count})) {
                             my %Groups = $Self->{GroupObject}->GroupList();
-                            foreach (@{$Self->{ConfigObject}->Get('UserSyncLDAPGroups')}) {
+                            foreach (@{$Self->{ConfigObject}->Get('UserSyncLDAPGroups'.$Self->{Count})}) {
                                 my $GroupID = '';
                                 foreach my $GID (keys %Groups) {
                                     if ($Groups{$GID} eq $_) {
@@ -337,32 +391,37 @@ sub Auth {
                         }
                     }
                     else {
-                       $Self->{LogObject}->Log(
-                           Priority => 'error',
-                           Message => "Can't create user '$Param{User}' ($UserDN) in RDBMS!",
-                       );
+                        $Self->{LogObject}->Log(
+                            Priority => 'error',
+                            Message => "Can't create user '$Param{User}' ($UserDN) in RDBMS!",
+                        );
                     }
                 }
                 else {
                     $Self->{UserObject}->UserUpdate(
-                        ID => $UserData{UserID},
-                        Salutation => 'Mr/Mrs',
-                        Login => $Param{User},
+                        %UserData,
+                        UserID => $UserData{UserID},
+                        UserLogin => $Param{User},
                         %SyncUser,
                         UserType => 'User',
-                        ValidID => 1,
-                        UserID => 1,
+                        ChangeUserID => 1,
                     );
                 }
             }
         }
         # sync ldap group 2 otrs group permissions
-        if ($Self->{ConfigObject}->Get('UserSyncLDAPGroupsDefination')) {
-            my $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+        if ($Self->{ConfigObject}->Get('UserSyncLDAPGroupsDefination'.$Self->{Count})) {
+            my $Result = '';
+            if ($Self->{SearchUserDN} && $Self->{SearchUserPw}) {
+                $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+            }
+            else {
+                $Result = $LDAP->bind();
+            }
             if ($Result->code) {
                 $Self->{LogObject}->Log(
-                  Priority => 'error',
-                  Message => "Sync bind failed! ".$Result->error,
+                    Priority => 'error',
+                    Message => "Sync bind failed! ".$Result->error,
                 );
                 # take down session
                 $LDAP->unbind;
@@ -372,7 +431,7 @@ sub Auth {
             my %UserData = $Self->{UserObject}->GetUserData(User => $Param{User});
             # system permissions
             my %PermissionsEmpty = ();
-            foreach (@{$Self->{ConfigObject}->Get('System::Permission')}) {
+            foreach (@{$Self->{ConfigObject}->Get('System::Permission'.$Self->{Count})}) {
                 $PermissionsEmpty{$_} = 0;
             }
             # remove all group permissions
@@ -388,7 +447,7 @@ sub Auth {
                 );
             }
             # group config settings
-            foreach my $GroupDN (sort keys %{$Self->{ConfigObject}->Get('UserSyncLDAPGroupsDefination')}) {
+            foreach my $GroupDN (sort keys %{$Self->{ConfigObject}->Get('UserSyncLDAPGroupsDefination'.$Self->{Count})}) {
                 # just in case for debug
                 $Self->{LogObject}->Log(
                     Priority => 'notice',
@@ -403,7 +462,7 @@ sub Auth {
                     $Filter = "($Self->{AccessAttr}=$Param{User})";
                 }
                 my $Result = $LDAP->search (
-                    base   => $GroupDN,
+                    base => $GroupDN,
                     filter => $Filter,
                 );
                 if ($Result->code) {
@@ -423,12 +482,12 @@ sub Auth {
                     $Self->{LogObject}->Log(
                         Priority => 'notice',
                         Message => "User: $Param{User} not in ".
-                          "GroupDN='$GroupDN', Filter='$Filter'! (REMOTE_ADDR: $RemoteAddr).",
+                            "GroupDN='$GroupDN', Filter='$Filter'! (REMOTE_ADDR: $RemoteAddr).",
                     );
                 }
                 else {
                     # sync groups permissions
-                    my %SGroups = %{$Self->{ConfigObject}->Get('UserSyncLDAPGroupsDefination')->{$GroupDN}};
+                    my %SGroups = %{$Self->{ConfigObject}->Get('UserSyncLDAPGroupsDefination'.$Self->{Count})->{$GroupDN}};
                     foreach my $SGroup (sort keys %SGroups) {
                         my %Permissions = %{$SGroups{$SGroup}};
                         # get group id
@@ -460,12 +519,18 @@ sub Auth {
             }
         }
         # sync ldap group 2 otrs role permissions
-        if ($Self->{ConfigObject}->Get('UserSyncLDAPRolesDefination')) {
-            my $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+        if ($Self->{ConfigObject}->Get('UserSyncLDAPRolesDefination'.$Self->{Count})) {
+            my $Result = '';
+            if ($Self->{SearchUserDN} && $Self->{SearchUserPw}) {
+                $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+            }
+            else {
+                $Result = $LDAP->bind();
+            }
             if ($Result->code) {
                 $Self->{LogObject}->Log(
-                  Priority => 'error',
-                  Message => "Sync bind failed! ".$Result->error,
+                    Priority => 'error',
+                    Message => "Sync bind failed! ".$Result->error,
                 );
                 # take down session
                 $LDAP->unbind;
@@ -484,7 +549,7 @@ sub Auth {
                 );
             }
             # group config settings
-            foreach my $GroupDN (sort keys %{$Self->{ConfigObject}->Get('UserSyncLDAPRolesDefination')}) {
+            foreach my $GroupDN (sort keys %{$Self->{ConfigObject}->Get('UserSyncLDAPRolesDefination'.$Self->{Count})}) {
                 # just in case for debug
                 $Self->{LogObject}->Log(
                     Priority => 'notice',
@@ -499,7 +564,7 @@ sub Auth {
                     $Filter = "($Self->{AccessAttr}=$Param{User})";
                 }
                 my $Result = $LDAP->search (
-                    base   => $GroupDN,
+                    base => $GroupDN,
                     filter => $Filter,
                 );
                 if ($Result->code) {
@@ -519,12 +584,12 @@ sub Auth {
                     $Self->{LogObject}->Log(
                         Priority => 'notice',
                         Message => "User: $Param{User} not in ".
-                          "GroupDN='$GroupDN', Filter='$Filter'! (REMOTE_ADDR: $RemoteAddr).",
+                            "GroupDN='$GroupDN', Filter='$Filter'! (REMOTE_ADDR: $RemoteAddr).",
                     );
                 }
                 else {
                     # sync groups permissions
-                    my %SRoles = %{$Self->{ConfigObject}->Get('UserSyncLDAPRolesDefination')->{$GroupDN}};
+                    my %SRoles = %{$Self->{ConfigObject}->Get('UserSyncLDAPRolesDefination'.$Self->{Count})->{$GroupDN}};
                     foreach my $SRole (sort keys %SRoles) {
                         # get group id
                         my $RoleID = '';
@@ -552,12 +617,18 @@ sub Auth {
             }
         }
         # sync ldap attribute 2 otrs group permissions
-        if ($Self->{ConfigObject}->Get('UserSyncLDAPAttibuteGroupsDefination')) {
-            my $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+        if ($Self->{ConfigObject}->Get('UserSyncLDAPAttibuteGroupsDefination'.$Self->{Count})) {
+            my $Result = '';
+            if ($Self->{SearchUserDN} && $Self->{SearchUserPw}) {
+                $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+            }
+            else {
+                $Result = $LDAP->bind();
+            }
             if ($Result->code) {
                 $Self->{LogObject}->Log(
-                  Priority => 'error',
-                  Message => "Sync bind failed! ".$Result->error,
+                    Priority => 'error',
+                    Message => "Sync bind failed! ".$Result->error,
                 );
                 # take down session
                 $LDAP->unbind;
@@ -567,7 +638,7 @@ sub Auth {
             my %UserData = $Self->{UserObject}->GetUserData(User => $Param{User});
             # system permissions
             my %PermissionsEmpty = ();
-            foreach (@{$Self->{ConfigObject}->Get('System::Permission')}) {
+            foreach (@{$Self->{ConfigObject}->Get('System::Permission'.$Self->{Count})}) {
                 $PermissionsEmpty{$_} = 0;
             }
             # remove all group permissions
@@ -586,7 +657,7 @@ sub Auth {
             my $Filter = "($Self->{UID}=$Param{User})";
             # perform search
             $Result = $LDAP->search (
-                base   => $Self->{BaseDN},
+                base => $Self->{BaseDN},
                 filter => $Filter,
             );
             if ($Result->code) {
@@ -595,7 +666,7 @@ sub Auth {
                     Message => "Search failed! (".$Self->{BaseDN}.") filter='$Filter' ".$Result->error,
                 );
             }
-            my %SyncConfig = %{$Self->{ConfigObject}->Get('UserSyncLDAPAttibuteGroupsDefination')};
+            my %SyncConfig = %{$Self->{ConfigObject}->Get('UserSyncLDAPAttibuteGroupsDefination'.$Self->{Count})};
             foreach my $Attribute (keys %SyncConfig) {
                 my %AttributeValues = %{$SyncConfig{$Attribute}};
                 foreach my $AttributeValue (keys %AttributeValues) {
@@ -636,12 +707,18 @@ sub Auth {
             }
         }
         # sync ldap attribute 2 otrs role permissions
-        if ($Self->{ConfigObject}->Get('UserSyncLDAPAttibuteRolesDefination')) {
-            my $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+        if ($Self->{ConfigObject}->Get('UserSyncLDAPAttibuteRolesDefination'.$Self->{Count})) {
+            my $Result = '';
+            if ($Self->{SearchUserDN} && $Self->{SearchUserPw}) {
+                $Result = $LDAP->bind(dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw});
+            }
+            else {
+                $Result = $LDAP->bind();
+            }
             if ($Result->code) {
                 $Self->{LogObject}->Log(
-                  Priority => 'error',
-                  Message => "Sync bind failed! ".$Result->error,
+                    Priority => 'error',
+                    Message => "Sync bind failed! ".$Result->error,
                 );
                 # take down session
                 $LDAP->unbind;
@@ -663,7 +740,7 @@ sub Auth {
             my $Filter = "($Self->{UID}=$Param{User})";
             # perform search
             $Result = $LDAP->search (
-                base   => $Self->{BaseDN},
+                base => $Self->{BaseDN},
                 filter => $Filter,
             );
             if ($Result->code) {
@@ -672,7 +749,7 @@ sub Auth {
                     Message => "Search failed! (".$Self->{BaseDN}.") filter='$Filter' ".$Result->error,
                 );
             }
-            my %SyncConfig = %{$Self->{ConfigObject}->Get('UserSyncLDAPAttibuteRolesDefination')};
+            my %SyncConfig = %{$Self->{ConfigObject}->Get('UserSyncLDAPAttibuteRolesDefination'.$Self->{Count})};
             foreach my $Attribute (keys %SyncConfig) {
                 my %AttributeValues = %{$SyncConfig{$Attribute}};
                 foreach my $AttributeValue (keys %AttributeValues) {
@@ -729,8 +806,8 @@ sub _ConvertTo {
     else {
         return $Self->{EncodeObject}->Convert(
             Text => $Text,
-            From => $Charset,
-            To => $Self->{DestCharset},
+            From => $Self->{DestCharset},
+            To => $Charset,
         );
     }
 }
