@@ -1,8 +1,8 @@
 # --
 # Kernel/System/DB.pm - the global database wrapper to support different databases
-# Copyright (C) 2001-2006 OTRS GmbH, http://otrs.org/
+# Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: DB.pm,v 1.55 2006/11/10 09:05:59 martin Exp $
+# $Id: DB.pm,v 1.73 2007/07/27 08:17:07 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -12,12 +12,13 @@
 package Kernel::System::DB;
 
 use strict;
+use warnings;
 use DBI;
 use Kernel::System::Time;
 use Kernel::System::Encode;
 
 use vars qw(@ISA $VERSION);
-$VERSION = '$Revision: 1.55 $';
+$VERSION = '$Revision: 1.73 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 =head1 NAME
@@ -38,29 +39,29 @@ All database functions to connect/insert/update/delete/... to a database.
 
 create database object with database connect
 
-  use Kernel::Config;
-  use Kernel::System::Log;
-  use Kernel::System::DB;
+    use Kernel::Config;
+    use Kernel::System::Log;
+    use Kernel::System::DB;
 
-  my $ConfigObject = Kernel::Config->new();
-  my $LogObject    = Kernel::System::Log->new(
-      ConfigObject => $ConfigObject,
-  );
+    my $ConfigObject = Kernel::Config->new();
+    my $LogObject = Kernel::System::Log->new(
+        ConfigObject => $ConfigObject,
+    );
 
-  $DBObject = Kernel::System::DB->new(
-      ConfigObject => $ConfigObject,
-      LogObject    => $LogObject,
-      # if you don't use the follow params, then this are used
-      # from Kernel/Config.pm!
-      DatabaseDSN  => 'DBI:odbc:database=123;host=localhost;',
-      DatabaseUser => 'user',
-      DatabasePw   => 'somepass',
-      Type         => 'mysql',
-      Attribute    => {
-          LongTruncOk => 1,
-          LongReadLen => 100*1024,
-      },
-  );
+    $DBObject = Kernel::System::DB->new(
+        ConfigObject => $ConfigObject,
+        LogObject => $LogObject,
+        # if you don't use the follow params, then this are used
+        # from Kernel/Config.pm!
+        DatabaseDSN => 'DBI:odbc:database=123;host=localhost;',
+        DatabaseUser => 'user',
+        DatabasePw => 'somepass',
+        Type => 'mysql',
+        Attribute => {
+            LongTruncOk => 1,
+            LongReadLen => 100*1024,
+        },
+    );
 
 =cut
 
@@ -76,7 +77,7 @@ sub new {
     $Self->{Debug} = $Param{Debug} || 0;
 
     # check needed objects
-    foreach (qw(ConfigObject LogObject)) {
+    foreach (qw(ConfigObject LogObject MainObject)) {
         if ($Param{$_}) {
             $Self->{$_} = $Param{$_};
         }
@@ -89,9 +90,10 @@ sub new {
     # time object
     $Self->{TimeObject} = Kernel::System::Time->new(%Param);
     # get config data
-    $Self->{DSN}  = $Param{DatabaseDSN} || $Self->{ConfigObject}->Get('DatabaseDSN');
+    $Self->{DSN} = $Param{DatabaseDSN} || $Self->{ConfigObject}->Get('DatabaseDSN');
     $Self->{USER} = $Param{DatabaseUser} || $Self->{ConfigObject}->Get('DatabaseUser');
-    $Self->{PW}   = $Param{DatabasePw} || $Self->{ConfigObject}->Get('DatabasePw');
+    $Self->{PW} = $Param{DatabasePw} || $Self->{ConfigObject}->Get('DatabasePw');
+    $Self->{SlowLog} = $Param{'Database::SlowLog'} || $Self->{ConfigObject}->Get('Database::SlowLog');
     # decrypt pw (if needed)
     if ($Self->{PW} =~ /^\{(.*)\}$/) {
         my $Length = length($1)*4;
@@ -137,12 +139,12 @@ sub new {
     # load backend module
     if ($Self->{'DB::Type'}) {
         my $GenericModule = "Kernel::System::DB::$Self->{'DB::Type'}";
-        if (!eval "require $GenericModule") {
-            die "Can't load database backend module $GenericModule! $@";
+        if (!$Self->{MainObject}->Require($GenericModule)) {
+            return;
         }
-        @ISA = ($GenericModule);
+        $Self->{Backend} = $GenericModule->new(%{$Self});
         # set database functions
-        $Self->LoadPreferences();
+        $Self->{Backend}->LoadPreferences();
     }
     else {
         $Self->{LogObject}->Log(
@@ -155,18 +157,19 @@ sub new {
 
     # check/get extra database config options
     # (overwrite auto detection with config options)
-    foreach (qw(Type Limit DirectBlob Attribute QuoteSingle QuoteBack)) {
+    foreach (qw(Type Limit DirectBlob Attribute QuoteSingle QuoteBack Connect Encode)) {
         if (defined($Self->{ConfigObject}->Get("Database::$_"))) {
-            $Self->{"DB::$_"} = $Self->{ConfigObject}->Get("Database::$_");
+            $Self->{Backend}->{"DB::$_"} = $Self->{ConfigObject}->Get("Database::$_");
         }
     }
     # check/get extra database config options
     # (overwrite with params)
-    foreach (qw(Type Limit DirectBlob Attribute QuoteSingle QuoteBack)) {
+    foreach (qw(Type Limit DirectBlob Attribute QuoteSingle QuoteBack Connect Encode)) {
         if (defined($Param{$_})) {
-            $Self->{"DB::$_"} = $Param{$_};
+            $Self->{Backend}->{"DB::$_"} = $Param{$_};
         }
     }
+
     # do database connect
     if (!$Self->Connect()) {
         return;
@@ -185,20 +188,23 @@ sub Connect {
     # debug
     if ($Self->{Debug} > 2) {
         $Self->{LogObject}->Log(
-            Caller   => 1,
+            Caller => 1,
             Priority => 'debug',
-            Message  => "DB.pm->Connect: DSN: $Self->{DSN}, User: $Self->{USER}, Pw: $Self->{PW}, DB Type: $Self->{'DB::Type'};",
+            Message => "DB.pm->Connect: DSN: $Self->{DSN}, User: $Self->{USER}, Pw: $Self->{PW}, DB Type: $Self->{'DB::Type'};",
         );
     }
     # db connect
-    if (!($Self->{dbh} = DBI->connect("$Self->{DSN}", $Self->{USER}, $Self->{PW}, $Self->{'DB::Attribute'}))) {
+    if (!($Self->{dbh} = DBI->connect("$Self->{DSN}", $Self->{USER}, $Self->{PW}, $Self->{Backend}->{'DB::Attribute'}))) {
         $Self->{LogObject}->Log(
-            Caller   => 1,
+            Caller => 1,
             Priority => 'Error',
-            Message  => $DBI::errstr,
+            Message => $DBI::errstr,
         );
         return;
     }
+    if ($Self->{Backend}->{"DB::Connect"}) {
+        $Self->Do(SQL => $Self->{Backend}->{"DB::Connect"});
+    }
     return $Self->{dbh};
 }
 
@@ -213,9 +219,9 @@ sub Disconnect {
     # debug
     if ($Self->{Debug} > 2) {
         $Self->{LogObject}->Log(
-            Caller   => 1,
+            Caller => 1,
             Priority => 'debug',
-            Message  => "DB.pm->Disconnect",
+            Message => "DB.pm->Disconnect",
         );
     }
     # do disconnect
@@ -227,19 +233,19 @@ sub Disconnect {
 
 to quote sql params
 
-  quote strings, date and time:
-  =============================
-  my $DBString = $DBObject->Quote("This isn't a problem!");
+    quote strings, date and time:
+    =============================
+    my $DBString = $DBObject->Quote("This isn't a problem!");
 
-  my $DBString = $DBObject->Quote("2005-10-27 20:15:01");
+    my $DBString = $DBObject->Quote("2005-10-27 20:15:01");
 
-  quote integers:
-  ===============
-  my $DBString = $DBObject->Quote(1234, 'Integer');
+    quote integers:
+    ===============
+    my $DBString = $DBObject->Quote(1234, 'Integer');
 
-  quote numbers (e. g. 1, 1.4, 42342.23424):
-  ==========================================
-  my $DBString = $DBObject->Quote(1234, 'Number');
+    quote numbers (e. g. 1, 1.4, 42342.23424):
+    ==========================================
+    my $DBString = $DBObject->Quote(1234, 'Number');
 
 =cut
 
@@ -267,9 +273,9 @@ sub Quote {
     elsif ($Type && $Type eq 'Number') {
         if ($Text !~ /^(\+|\-|)(\d{1,20}|\d{1,20}\.\d{1,20})$/) {
             $Self->{LogObject}->Log(
-                Caller   => 1,
+                Caller => 1,
                 Priority => 'error',
-                Message  => "Invalid number in query '$Text'!",
+                Message => "Invalid number in query '$Text'!",
             );
             return '';
         }
@@ -277,16 +283,7 @@ sub Quote {
     }
     # do quote string
     elsif (!defined($Type)) {
-        if ($Self->{'DB::QuoteBack'}) {
-            $Text =~ s/\\/$Self->{'DB::QuoteBack'}\\/g;
-        }
-        if ($Self->{'DB::QuoteSingle'}) {
-            $Text =~ s/'/$Self->{'DB::QuoteSingle'}'/g;
-        }
-        if ($Self->{'DB::QuoteSemicolon'}) {
-            $Text =~ s/;/$Self->{'DB::QuoteSemicolon'};/g;
-        }
-        return $Text;
+        return ${$Self->{Backend}->Quote(\$Text)};
     }
     else {
         $Self->{LogObject}->Log(
@@ -302,7 +299,7 @@ sub Quote {
 
 to get database errors back
 
-  my $ErrorMessage = $DBObject->Error();
+    my $ErrorMessage = $DBObject->Error();
 
 =cut
 
@@ -315,19 +312,19 @@ sub Error {
 
 to insert, update or delete something
 
-  $DBObject->Do(SQL => "INSERT INTO table (name) VALUES ('dog')");
+    $DBObject->Do(SQL => "INSERT INTO table (name) VALUES ('dog')");
 
-  $DBObject->Do(SQL => "DELETE FROM table");
+    $DBObject->Do(SQL => "DELETE FROM table");
 
-  you also can use DBI bind values (used for large strings):
+    you also can use DBI bind values (used for large strings):
 
-  my $Var1 = 'dog1';
-  my $Var2 = 'dog2';
+    my $Var1 = 'dog1';
+    my $Var2 = 'dog2';
 
-  $DBObject->Do(
-      SQL => "INSERT INTO table (name1, name2) VALUES (?, ?)",
-      Bind => [\$Var1, \$Var2],
-  );
+    $DBObject->Do(
+        SQL => "INSERT INTO table (name1, name2) VALUES (?, ?)",
+        Bind => [\$Var1, \$Var2],
+    );
 
 =cut
 
@@ -358,8 +355,8 @@ sub Do {
     }
     if (!$Self->{ConfigObject}->Get('TimeZone')) {
         # doing timestamp workaround (if needed)
-        if ($Self->{'DB::CurrentTimestamp'}) {
-            $Param{SQL} =~ s/current_timestamp/$Self->{'DB::CurrentTimestamp'}/g;
+        if ($Self->{Backend}->{'DB::CurrentTimestamp'}) {
+            $Param{SQL} =~ s/current_timestamp/$Self->{Backend}->{'DB::CurrentTimestamp'}/g;
         }
     }
     else {
@@ -371,17 +368,27 @@ sub Do {
     if ($Self->{Debug} > 0) {
         $Self->{DoCounter}++;
         $Self->{LogObject}->Log(
-            Caller   => 1,
+            Caller => 1,
             Priority => 'debug',
-            Message  => "DB.pm->Do ($Self->{DoCounter}) SQL: '$Param{SQL}'",
+            Message => "DB.pm->Do ($Self->{DoCounter}) SQL: '$Param{SQL}'",
         );
     }
+    # check length, don't use more the 4 k
+    use bytes;
+    if (length($Param{SQL}) > 4*1024) {
+        $Self->{LogObject}->Log(
+            Caller => 1,
+            Priority => 'error',
+            Message => "Your SQL is longer the 4k, this probably not work on many databases (use Bind instead)!",
+        );
+    }
+    no bytes;
     # send sql to database
     if (!$Self->{dbh}->do($Param{SQL}, undef, @Array)) {
         $Self->{LogObject}->Log(
-            Caller   => 1,
+            Caller => 1,
             Priority => 'Error',
-            Message  => "$DBI::errstr, SQL: '$Param{SQL}'",
+            Message => "$DBI::errstr, SQL: '$Param{SQL}'",
         );
         return;
     }
@@ -392,10 +399,26 @@ sub Do {
 
 to send a select something to the database
 
-  $DBObject->Prepare(
-      SQL => "SELECT id, name FROM table",
-      Limit => 10
-  );
+    $DBObject->Prepare(
+        SQL => "SELECT id, name FROM table",
+        Limit => 10,
+    );
+
+or in case you want just to get row 10 till 30
+
+    $DBObject->Prepare(
+        SQL => "SELECT id, name FROM table",
+        Start => 10,
+        Limit => 20,
+    );
+
+in case you wan't not utf-8 encoding for some column use this to do
+not encode content column
+
+    $DBObject->Prepare(
+        SQL => "SELECT id, name, content FROM table",
+        Encode => [1,1,0],
+    );
 
 =cut
 
@@ -404,19 +427,31 @@ sub Prepare {
     my %Param = @_;
     my $SQL = $Param{SQL};
     my $Limit = $Param{Limit} || '';
+    my $Start = $Param{Start} || '';
     # check needed stuff
     if (!$Param{SQL}) {
         $Self->{LogObject}->Log(Priority => 'error', Message => "Need SQL!");
         return;
     }
+    if (defined($Param{Encode})) {
+        $Self->{Encode} = $Param{Encode};
+    }
+    else {
+        $Self->{Encode} = undef;
+    }
     $Self->{Limit} = 0;
+    $Self->{LimitStart} = 0;
     $Self->{LimitCounter} = 0;
     # build finally select query
     if ($Limit) {
-        if ($Self->{'DB::Limit'} eq 'limit') {
+        if ($Start) {
+            $Limit = $Limit + $Start;
+            $Self->{LimitStart} = $Start;
+        }
+        if ($Self->{Backend}->{'DB::Limit'} eq 'limit') {
             $SQL .= " LIMIT $Limit";
         }
-        elsif ($Self->{'DB::Limit'} eq 'fetch') {
+        elsif ($Self->{Backend}->{'DB::Limit'} eq 'fetch') {
             $SQL .= " fetch $Limit first row";
         }
         else {
@@ -432,6 +467,11 @@ sub Prepare {
             Message => "DB.pm->Prepare ($Self->{PrepareCounter}/".time().") SQL: '$SQL'",
         );
     }
+    # slow log feature
+    my $LogTime;
+    if ($Self->{SlowLog}) {
+        $LogTime = time();
+    }
     # do
     if (!($Self->{Curser} = $Self->{dbh}->prepare($SQL))) {
         $Self->{LogObject}->Log(
@@ -449,6 +489,17 @@ sub Prepare {
         );
         return;
     }
+    # slow log feature
+    if ($Self->{SlowLog}) {
+        my $LogTimeTaken = time() - $LogTime;
+        if ($LogTimeTaken > 4) {
+            $Self->{LogObject}->Log(
+                Caller => 1,
+                Priority => 'error',
+                Message => "Slow ($LogTimeTaken s) SQL: '$SQL'",
+            );
+        }
+    }
     return 1;
 }
 
@@ -456,32 +507,49 @@ sub Prepare {
 
 to get a select return
 
-  $DBObject->Prepare(
-      SQL => "SELECT id, name FROM table",
-      Limit => 10
-  );
+    $DBObject->Prepare(
+        SQL => "SELECT id, name FROM table",
+        Limit => 10
+    );
 
-  while (my @Row = $DBObject->FetchrowArray()) {
-      print "$Row[0]:$Row[1]\n";
-  }
+    while (my @Row = $DBObject->FetchrowArray()) {
+        print "$Row[0]:$Row[1]\n";
+    }
 
 =cut
 
 sub FetchrowArray {
     my $Self = shift;
     # work with cursers if database don't support limit
-    if (!$Self->{'DB::Limit'} && $Self->{Limit}) {
+    if (!$Self->{Backend}->{'DB::Limit'} && $Self->{Limit}) {
         if ($Self->{Limit} <= $Self->{LimitCounter}) {
             $Self->{Curser}->finish();
             return;
         }
         $Self->{LimitCounter}++;
     }
+    # fetch first not used rows
+    if ($Self->{LimitStart}) {
+        foreach (1..$Self->{LimitStart}) {
+            if (!$Self->{Curser}->fetchrow_array()) {
+                $Self->{LimitStart} = 0;
+                return ();
+            }
+            $Self->{LimitCounter}++;
+        }
+        $Self->{LimitStart} = 0;
+    }
     # return
     my @Row = $Self->{Curser}->fetchrow_array();
     # e. g. set utf-8 flag
+    my $Count = 0;
     foreach (@Row) {
-        $Self->{EncodeObject}->Encode(\$_);
+        if ($Self->{Backend}->{"DB::Encode"}) {
+            if (!defined($Self->{Encode}) || ($Self->{Encode} && $Self->{Encode}->[$Count])) {
+                $Self->{EncodeObject}->Encode(\$_);
+            }
+        }
+        $Count++;
     }
     return @Row;
 }
@@ -490,13 +558,21 @@ sub FetchrowArray {
 sub FetchrowHashref {
     my $Self = shift;
     # work with cursers if database don't support limit
-    if (!$Self->{'DB::Limit'} && $Self->{Limit}) {
+    if (!$Self->{Backend}->{'DB::Limit'} && $Self->{Limit}) {
         if ($Self->{Limit} <= $Self->{LimitCounter}) {
             $Self->{Curser}->finish();
+            $Self->{LimitCounter}++;
             return;
         }
         $Self->{LimitCounter}++;
     }
+    # fetch first not used rows
+    if ($Self->{LimitStart}) {
+        foreach (1..$Self->{LimitStart}) {
+            $Self->{Curser}->fetchrow_array();
+        }
+        $Self->{LimitStart} = 0;
+    }
     # return
     return $Self->{Curser}->fetchrow_hashref();
 }
@@ -505,41 +581,41 @@ sub FetchrowHashref {
 
 to get database functions like Limit, DirectBlob, ...
 
-  my $What = $DBObject->GetDatabaseFunction('DirectBlob');
+    my $What = $DBObject->GetDatabaseFunction('DirectBlob');
 
 =cut
 
 sub GetDatabaseFunction {
     my $Self = shift;
     my $What = shift;
-    return $Self->{'DB::'.$What};
+    return $Self->{Backend}->{'DB::'.$What};
 }
 
 =item SQLProcessor()
 
 generate database based sql syntax (e. g. CREATE TABLE ...)
 
-  my @SQL = $DBObject->SQLProcessor(
-    Database =>
-      [
-         Tag => 'TableCreate',
-         Name => 'table_name',
-      ],
-      [
-         Tag => 'Column',
-         Name => 'col_name',
-         Type => 'VARCHAR',
-         Size => 150,
-      ],
-      [
-         Tag => 'Column',
-         Name => 'col_name2',
-         Type => 'INTERGER',
-      ],
-      [
-         Tag => 'TableEnd',
-      ],
-  );
+    my @SQL = $DBObject->SQLProcessor(
+        Database =>
+            [
+                Tag => 'TableCreate',
+                Name => 'table_name',
+            ],
+            [
+                Tag => 'Column',
+                Name => 'col_name',
+                Type => 'VARCHAR',
+                Size => 150,
+            ],
+            [
+                Tag => 'Column',
+                Name => 'col_name2',
+                Type => 'INTEGER',
+            ],
+            [
+                Tag => 'TableEnd',
+            ],
+    );
 
 =cut
 
@@ -556,7 +632,7 @@ sub SQLProcessor {
             }
             elsif (($Tag->{Tag} eq 'Table' || $Tag->{Tag} eq 'TableCreate') && $Tag->{TagType} eq 'End') {
                 push (@Table, $Tag);
-                push (@SQL, $Self->TableCreate(@Table));
+                push (@SQL, $Self->{Backend}->TableCreate(@Table));
                 @Table = ();
             }
             elsif ($Tag->{Tag} eq 'Column' && $Tag->{TagType} eq 'Start') {
@@ -584,7 +660,7 @@ sub SQLProcessor {
             }
             elsif ($Tag->{Tag} eq 'TableDrop' && $Tag->{TagType} eq 'Start') {
                 push (@Table, $Tag);
-                push (@SQL, $Self->TableDrop(@Table));
+                push (@SQL, $Self->{Backend}->TableDrop(@Table));
                 @Table = ();
             }
             elsif ($Tag->{Tag} eq 'TableAlter' && $Tag->{TagType} eq 'Start') {
@@ -607,7 +683,7 @@ sub SQLProcessor {
             }
             elsif ($Tag->{Tag} eq 'TableAlter' && $Tag->{TagType} eq 'End') {
                 push (@Table, $Tag);
-                push (@SQL, $Self->TableAlter(@Table));
+                push (@SQL, $Self->{Backend}->TableAlter(@Table));
                 @Table = ();
             }
             elsif ($Tag->{Tag} eq 'Insert' && $Tag->{TagType} eq 'Start') {
@@ -618,7 +694,7 @@ sub SQLProcessor {
             }
             elsif ($Tag->{Tag} eq 'Insert' && $Tag->{TagType} eq 'End') {
                 push (@Table, $Tag);
-                push (@SQL, $Self->Insert(@Table));
+                push (@SQL, $Self->{Backend}->Insert(@Table));
                 @Table = ();
             }
         }
@@ -631,7 +707,7 @@ sub SQLProcessor {
 generate database based sql syntax, post data of SQLProcessor(),
 e. g. foreign keys
 
-  my @SQL = $DBObject->SQLProcessorPost();
+    my @SQL = $DBObject->SQLProcessorPost();
 
 =cut
 
@@ -639,9 +715,9 @@ sub SQLProcessorPost {
     my $Self = shift;
     my %Param = @_;
     my @SQL = ();
-    if ($Self->{Post}) {
-        my @Return = @{$Self->{Post}};
-        undef $Self->{Post};
+    if ($Self->{Backend}->{Post}) {
+        my @Return = @{$Self->{Backend}->{Post}};
+        undef $Self->{Backend}->{Post};
         return @Return;
     }
     else {
@@ -649,29 +725,36 @@ sub SQLProcessorPost {
     }
 }
 
-=item GetTableData()
-
-to get table data back in a hash
-
-  my %Users = $DBObject->GetTableData(
-      What => 'id, name',
-      Table => 'groups',
-  );
-
-=cut
+# GetTableData()
+#
+# !! DONT USE THIS FUNCTION !!
+#
+# Due to compatibility reason this function is still in use and will be removed
+# in a further release.
 
 sub GetTableData {
     my $Self = shift;
     my %Param = @_;
     my $Table = $Param{Table};
     my $What = $Param{What};
-    my $Whare = $Param{Where} || '';
+    my $Where = $Param{Where} || '';
     my $Valid = $Param{Valid} || '';
     my $Clamp = $Param{Clamp} || '';
     my %Data;
+
     my $SQL = "SELECT $What FROM $Table ";
-    $SQL .= " WHERE " . $Whare if ($Whare);
-    $SQL .= " WHERE valid_id IN ( ${\(join ', ', $Self->GetValidIDs())} )" if ((!$Whare) && ($Valid));
+    $SQL .= " WHERE " . $Where if ($Where);
+
+    if (!$Where && $Valid) {
+        my @ValidIDs;
+
+        $Self->Prepare(SQL => "SELECT id FROM valid WHERE name = 'valid'");
+        while (my @Row = $Self->FetchrowArray()) {
+            push(@ValidIDs, $Row[0]);
+        }
+
+        $SQL .= " WHERE valid_id IN ( ${\(join ', ', @ValidIDs)} )";
+    }
     $Self->Prepare(SQL => $SQL);
     while (my @Row = $Self->FetchrowArray()) {
         if ($Row[3]) {
@@ -697,24 +780,6 @@ sub GetTableData {
     return %Data;
 }
 
-sub GetValidIDs {
-    my $Self = shift;
-    my %Param = @_;
-    my @ValidIDs;
-    if ($Self->{ValidIDs}) {
-        my $ValidIDsTmp = $Self->{ValidIDs};
-        @ValidIDs = @$ValidIDsTmp;
-    }
-    else {
-        $Self->Prepare(SQL => "SELECT id FROM valid WHERE name = 'valid'");
-        while (my @RowTmp = $Self->FetchrowArray()) {
-            push(@ValidIDs, $RowTmp[0]);
-        }
-        $Self->{ValidIDs} = \@ValidIDs;
-    }
-    return @ValidIDs;
-}
-
 sub _TypeCheck {
     my $Self = shift;
     my $Tag = shift;
@@ -733,6 +798,8 @@ sub DESTROY {
 }
 1;
 
+=back
+
 =head1 TERMS AND CONDITIONS
 
 This software is part of the OTRS project (http://otrs.org/).
@@ -745,6 +812,6 @@ did not receive this file, see http://www.gnu.org/licenses/gpl.txt.
 
 =head1 VERSION
 
-$Revision: 1.55 $ $Date: 2006/11/10 09:05:59 $
+$Revision: 1.73 $ $Date: 2007/07/27 08:17:07 $
 
 =cut
