@@ -2,7 +2,7 @@
 # Kernel/System/Ticket/Article.pm - global article module for OTRS kernel
 # Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: Article.pm,v 1.128.2.4 2007/03/02 01:19:21 martin Exp $
+# $Id: Article.pm,v 1.144 2007/07/26 12:56:12 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -18,7 +18,7 @@ use Mail::Internet;
 use Kernel::System::StdAttachment;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.128.2.4 $';
+$VERSION = '$Revision: 1.144 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 =head1 NAME
@@ -39,25 +39,24 @@ All article functions.
 
 create an article
 
-  my $ArticleID = $TicketObject->ArticleCreate(
-      TicketID => 123,
-      ArticleType => 'note-internal', # email-external|email-internal|phone|fax|...
-      SenderType => 'agent',         # agent|system|customer
-      From => 'Some Agent <email@example.com>',   # not required but useful
-      To => 'Some Customer A <customer-a@example.com>', # not required but useful
-      Cc => 'Some Customer B <customer-b@example.com>', # not required but useful
-      ReplyTo => 'Some Customer B <customer-b@example.com>', # not required
-      Subject => 'some short description',        # required
-      Body => 'the message text',                 # required
-      MessageID => '<asdasdasd.123@example.com>', # not required but useful
-      ContentType => 'text/plain; charset=ISO-8859-15',
-      HistoryType => 'OwnerUpdate',  # EmailCustomer|Move|AddNote|PriorityUpdate|WebRequestCustomer|...
-      HistoryComment => 'Some free text!',
-      UserID => 123,
-
-      NoAgentNotify => 0,            # if you don't want to send agent notifications
-      ForceNotificationToUserID => [1,43,56],     # if you want to force somebody
-  );
+    my $ArticleID = $TicketObject->ArticleCreate(
+        TicketID => 123,
+        ArticleType => 'note-internal', # email-external|email-internal|phone|fax|...
+        SenderType => 'agent',         # agent|system|customer
+        From => 'Some Agent <email@example.com>',   # not required but useful
+        To => 'Some Customer A <customer-a@example.com>', # not required but useful
+        Cc => 'Some Customer B <customer-b@example.com>', # not required but useful
+        ReplyTo => 'Some Customer B <customer-b@example.com>', # not required
+        Subject => 'some short description',        # required
+        Body => 'the message text',                 # required
+        MessageID => '<asdasdasd.123@example.com>', # not required but useful
+        ContentType => 'text/plain; charset=ISO-8859-15',
+        HistoryType => 'OwnerUpdate',  # EmailCustomer|Move|AddNote|PriorityUpdate|WebRequestCustomer|...
+        HistoryComment => 'Some free text!',
+        UserID => 123,
+        NoAgentNotify => 0,            # if you don't want to send agent notifications
+        ForceNotificationToUserID => [1,43,56],     # if you want to force somebody
+    );
 
 =cut
 
@@ -130,12 +129,14 @@ sub ArticleCreate {
         " valid_id, incoming_time,  create_time, create_by, change_time, change_by) " .
         " VALUES ".
         " ($DBParam{TicketID}, $DBParam{ArticleTypeID}, $DBParam{SenderTypeID}, ".
-        " '$DBParam{From}', '$DBParam{ReplyTo}', '$DBParam{To}', '$DBParam{Cc}', ".
-        " '$DBParam{Subject}', ".
+        " '$DBParam{From}', '$DBParam{ReplyTo}', ?, ?, ?, ".
         " '$DBParam{MessageID}', ?, '$DBParam{ContentType}', ?, ".
         " $ValidID,  $IncomingTime, " .
         " current_timestamp, $DBParam{UserID}, current_timestamp, $DBParam{UserID})";
-    if (!$Self->{DBObject}->Do(SQL => $SQL, Bind => [\$Param{Body}, \$Self->{ArticleContentPath}])) {
+    if (!$Self->{DBObject}->Do(
+        SQL => $SQL,
+        Bind => [\$Param{To}, \$Param{Cc}, \$Param{Subject}, \$Param{Body}, \$Self->{ArticleContentPath}])
+    ) {
         return;
     }
     # get article id
@@ -214,7 +215,7 @@ sub ArticleCreate {
                 EscalationStartTime => $Self->{TimeObject}->SystemTime(),
                 TicketID => $Param{TicketID},
                 UserID => $Param{UserID},
-           );
+            );
             $Self->TicketUnlockTimeoutUpdate(
                  UnlockTimeout => $Self->{TimeObject}->SystemTime(),
                  TicketID => $Param{TicketID},
@@ -222,17 +223,22 @@ sub ArticleCreate {
             );
         }
     }
+    # check if latest article is sent to customer
     elsif ($Param{SenderType} eq 'agent' && $Param{ArticleType} =~ /email-ext|phone|fax|sms|note-ext/) {
-        # check if latest article is sent to customer
+        $Self->TicketEscalationResponseTimeUpdate(
+            EscalationResponseTime => $Self->{TimeObject}->SystemTime(),
+            TicketID => $Param{TicketID},
+            UserID => $Param{UserID},
+        );
         $Self->TicketEscalationStartUpdate(
             EscalationStartTime => $Self->{TimeObject}->SystemTime(),
             TicketID => $Param{TicketID},
             UserID => $Param{UserID},
         );
         $Self->TicketUnlockTimeoutUpdate(
-             UnlockTimeout => $Self->{TimeObject}->SystemTime(),
-             TicketID => $Param{TicketID},
-             UserID => $Param{UserID},
+            UnlockTimeout => $Self->{TimeObject}->SystemTime(),
+            TicketID => $Param{TicketID},
+            UserID => $Param{UserID},
         );
     }
     # send auto response
@@ -271,7 +277,7 @@ sub ArticleCreate {
                 $Self->{LogObject}->Log(
                     Priority => 'notice',
                     Message => "Sent no '$Param{AutoResponseType}' for Ticket [".
-                      "$Ticket{TicketNumber}] ($OrigHeader{From}) "
+                        "$Ticket{TicketNumber}] ($OrigHeader{From}) "
                 );
             }
             else {
@@ -391,6 +397,11 @@ sub ArticleCreate {
             }
             else {
                 @OwnerIDs = $Self->GetSubscribedUserIDsByQueueID(QueueID => $Ticket{QueueID});
+                foreach (qw(OwnerID ResponsibleID)) {
+                    if ($Ticket{$_}) {
+                        push (@OwnerIDs, $Ticket{$_});
+                    }
+                }
             }
             foreach (@OwnerIDs) {
                 if (!$AlreadySent{$_}) {
@@ -481,6 +492,31 @@ sub ArticleCreate {
             }
         }
     }
+    # update note to: field
+    if (%AlreadySent) {
+        if (!$Param{ArticleType}) {
+            $Param{ArticleType} = $Self->ArticleTypeLookup(ArticleTypeID => $Param{ArticleTypeID});
+        }
+        if ($Param{ArticleType} =~ /^note\-/ && $Param{UserID} ne 1) {
+            my $NewTo = $Param{To} || '';
+            foreach my $UserID (keys %AlreadySent) {
+                my %UserData = $Self->{UserObject}->GetUserData(
+                    UserID => $UserID,
+                    Cached => 1,
+                    Valid => 1,
+                );
+                if ($NewTo) {
+                    $NewTo .= ", ";
+                }
+                $NewTo .= "$UserData{UserFirstname} $UserData{UserLastname} <$UserData{UserEmail}>";
+            }
+            if ($NewTo) {
+                $Self->{DBObject}->Do(
+                    SQL => "UPDATE article SET a_to = '".$Self->{DBObject}->Quote($NewTo)."' WHERE id = $ArticleID",
+                );
+            }
+        }
+    }
     # return ArticleID
     return $ArticleID;
 }
@@ -532,9 +568,9 @@ sub _ArticleGetId {
 
 get ticket id of given message id
 
-  my $TicketID = $TicketObject->ArticleGetTicketIDOfMessageID(
-      MessageID=> '<13231231.1231231.32131231@example.com>',
-  );
+    my $TicketID = $TicketObject->ArticleGetTicketIDOfMessageID(
+        MessageID=> '<13231231.1231231.32131231@example.com>',
+    );
 
 =cut
 
@@ -583,9 +619,9 @@ sub ArticleGetTicketIDOfMessageID {
 
 get article content path
 
-  my $Path = $TicketObject->ArticleGetContentPath(
-      ArticleID => 123,
-  );
+    my $Path = $TicketObject->ArticleGetContentPath(
+        ArticleID => 123,
+    );
 
 =cut
 
@@ -622,13 +658,13 @@ sub ArticleGetContentPath {
 
 article sender lookup
 
-  my $SenderTypeID = $TicketObject->ArticleSenderTypeLookup(
-      SenderType => 'customer', # customer|system|agent
-  );
+    my $SenderTypeID = $TicketObject->ArticleSenderTypeLookup(
+        SenderType => 'customer', # customer|system|agent
+    );
 
-  my $SenderType = $TicketObject->ArticleSenderTypeLookup(
-      SenderTypeID => 1,
-  );
+    my $SenderType = $TicketObject->ArticleSenderTypeLookup(
+        SenderTypeID => 1,
+    );
 
 =cut
 
@@ -688,13 +724,13 @@ sub ArticleSenderTypeLookup {
 
 article type lookup
 
-  my $ArticleTypeID = $TicketObject->ArticleTypeLookup(
-      ArticleType => 'webrequest-customer', # note-internal|...
-  );
+    my $ArticleTypeID = $TicketObject->ArticleTypeLookup(
+        ArticleType => 'webrequest-customer', # note-internal|...
+    );
 
-  my $ArticleType = $TicketObject->ArticleTypeLookup(
-      ArticleTypeID => 1,
-  );
+    my $ArticleType = $TicketObject->ArticleTypeLookup(
+        ArticleTypeID => 1,
+    );
 
 =cut
 
@@ -755,12 +791,12 @@ sub ArticleTypeLookup {
 
 get a article type list
 
-  my @ArticleTypeList = $TicketObject->ArticleTypesList();
+    my @ArticleTypeList = $TicketObject->ArticleTypeList();
 
-  # to get just customer shown article types
-  my @ArticleTypeList = $TicketObject->ArticleTypesList(
-      Type => 'Customer',
-  );
+    # to get just customer shown article types
+    my @ArticleTypeList = $TicketObject->ArticleTypesList(
+        Type => 'Customer',
+    );
 
 =cut
 
@@ -776,16 +812,16 @@ sub ArticleTypeList {
         }
     }
     my $SQL = "SELECT id, name FROM article_type ".
-        " WHERE valid_id IN (${\(join ', ', $Self->{DBObject}->GetValidIDs())}) ";
+        " WHERE valid_id IN (${\(join ', ', $Self->{ValidObject}->ValidIDsGet())}) ";
     $Self->{DBObject}->Prepare(SQL => $SQL);
-    while (my @Row = $Self->{DBObject}->FetchrowArray()){
+    while (my @Row = $Self->{DBObject}->FetchrowArray()) {
         if ($Param{Type} && $Param{Type} eq 'Customer') {
-           if ($Row[1] !~ /int/i){
-              push (@List, $Row[1]);
-           }
+            if ($Row[1] !~ /int/i) {
+                push (@List, $Row[1]);
+            }
         }
         else {
-           push (@List, $Row[1]);
+            push (@List, $Row[1]);
         }
     }
     return @List;
@@ -798,23 +834,23 @@ get _possible_ article free text options
 
 Note: the current value is accessible over ArticleGet()
 
-  my $HashRef = $TicketObject->ArticleFreeTextGet(
-     Type => 'ArticleFreeText3',
-     ArticleID => 123,
-     UserID => 123, # or CustomerUserID
-  );
+    my $HashRef = $TicketObject->ArticleFreeTextGet(
+        Type => 'ArticleFreeText3',
+        ArticleID => 123,
+        UserID => 123, # or CustomerUserID
+    );
 
-  my $HashRef = $TicketObject->ArticleFreeTextGet(
-     Type => 'ArticleFreeText3',
-     UserID => 123, # or CustomerUserID
-  );
+    my $HashRef = $TicketObject->ArticleFreeTextGet(
+        Type => 'ArticleFreeText3',
+        UserID => 123, # or CustomerUserID
+    );
 
-  # fill up with existing values
-  my $HashRef = $TicketObject->ArticleFreeTextGet(
-     Type => 'ArticleFreeText3',
-     FillUp => 1,
-     UserID => 123, # or CustomerUserID
-  );
+    # fill up with existing values
+    my $HashRef = $TicketObject->ArticleFreeTextGet(
+        Type => 'ArticleFreeText3',
+        FillUp => 1,
+        UserID => 123, # or CustomerUserID
+    );
 
 =cut
 
@@ -883,14 +919,14 @@ sub ArticleFreeTextGet {
 
 set article free text
 
-  $TicketObject->ArticleFreeTextSet(
-      TicketID => 123,
-      ArticleID => 1234,
-      Counter => 1,
-      Key => 'Planet',
-      Value => 'Sun',
-      UserID => 123,
-  );
+    $TicketObject->ArticleFreeTextSet(
+        TicketID => 123,
+        ArticleID => 1234,
+        Counter => 1,
+        Key => 'Planet',
+        Value => 'Sun',
+        UserID => 123,
+    );
 
 =cut
 
@@ -934,9 +970,9 @@ sub ArticleFreeTextSet {
 
 get last customer article
 
-  my %Article = $TicketObject->ArticleLastCustomerArticle(
-      TicketID => 123,
-  );
+    my %Article = $TicketObject->ArticleLastCustomerArticle(
+        TicketID => 123,
+    );
 
 =cut
 
@@ -984,9 +1020,9 @@ sub ArticleLastCustomerArticle {
 
 get first article
 
-  my %Article = $TicketObject->ArticleFirstArticle(
-      TicketID => 123,
-  );
+    my %Article = $TicketObject->ArticleFirstArticle(
+        TicketID => 123,
+    );
 
 =cut
 
@@ -1017,14 +1053,14 @@ sub ArticleFirstArticle {
 
 returns an array with article id's
 
-  my @ArticleIDs = $TicketObject->ArticleIndex(
-      TicketID => 123,
-  );
+    my @ArticleIDs = $TicketObject->ArticleIndex(
+        TicketID => 123,
+    );
 
-  my @ArticleIDs = $TicketObject->ArticleIndex(
-      SenderType => 'customer',
-      TicketID => 123,
-  );
+    my @ArticleIDs = $TicketObject->ArticleIndex(
+        SenderType => 'customer',
+        TicketID => 123,
+    );
 
 =cut
 
@@ -1077,9 +1113,9 @@ sub ArticleIndex {
 
 returns an array with hash ref
 
-  my @ArticleIDs = $TicketObject->ArticleContentIndex(
-      TicketID => 123,
-  );
+    my @ArticleIDs = $TicketObject->ArticleContentIndex(
+        TicketID => 123,
+    );
 
 =cut
 
@@ -1091,12 +1127,6 @@ sub ArticleContentIndex {
         $Self->{LogObject}->Log(Priority => 'error', Message => "Need TicketID!");
         return;
     }
-#    my @ArticleIndex = $Self->ArticleIndex(TicketID => $Param{TicketID});
-#    my @ArticleBox = ();
-#    foreach (@ArticleIndex) {
-#        my %Article = $Self->ArticleGet(ArticleID => $_);
-#        push (@ArticleBox, \%Article);
-#    }
     my @ArticleBox = $Self->ArticleGet(TicketID => $Param{TicketID});
     # article attachments
     foreach my $Article (@ArticleBox) {
@@ -1113,13 +1143,13 @@ sub ArticleContentIndex {
 
 returns article data
 
-  my %Article = $TicketObject->ArticleGet(
-      ArticleID => 123,
-  );
+    my %Article = $TicketObject->ArticleGet(
+        ArticleID => 123,
+    );
 
-  my @ArticleIndex = $TicketObject->ArticleGet(
-      TicketID => 123,
-  );
+    my @ArticleIndex = $TicketObject->ArticleGet(
+        TicketID => 123,
+    );
 
 =cut
 
@@ -1153,7 +1183,7 @@ sub ArticleGet {
     # sql query
     my @Content = ();
     my $SQL = "SELECT sa.ticket_id, sa.a_from, sa.a_to, sa.a_cc, sa.a_subject, ".
-        " sa.a_reply_to, sa. a_message_id, sa.a_body, ".
+        " sa.a_reply_to, sa.a_message_id, sa.a_body, ".
         " st.create_time_unix, st.ticket_state_id, st.queue_id, sa.create_time, ".
         " sa.a_content_type, sa.create_by, st.tn, article_sender_type_id, st.customer_id, ".
         " st.until_time, st.ticket_priority_id, st.customer_user_id, st.user_id, ".
@@ -1169,7 +1199,8 @@ sub ArticleGet {
         " st.freekey13, st.freetext13, st.freekey14, st.freetext14, ".
         " st.freekey15, st.freetext15, st.freekey16, st.freetext16, ".
         " st.ticket_lock_id, st.title, st.escalation_start_time, ".
-        " st.freetime1 , st.freetime2 ".
+        " st.freetime1 , st.freetime2, st.freetime3, st.freetime4, st.freetime5, st.freetime6, ".
+        " st.type_id, st.service_id, st.sla_id, st.escalation_response_time, st.escalation_solution_time ".
         " FROM ".
         " article sa, ticket st ".
         " WHERE ";
@@ -1198,6 +1229,10 @@ sub ArticleGet {
         $Ticket{Title} = $Data{Title};
         $Data{EscalationStartTime} = $Row[66];
         $Ticket{EscalationStartTime} = $Data{EscalationStartTime};
+        $Data{EscalationResponseTime} = $Row[76];
+        $Ticket{EscalationResponseTime} = $Data{EscalationResponseTime};
+        $Data{EscalationSolutionTime} = $Row[77];
+        $Ticket{EscalationSolutionTime} = $Data{EscalationSolutionTime};
         $Data{From} = $Row[1];
         $Data{To} = $Row[2];
         $Data{Cc} = $Row[3];
@@ -1207,6 +1242,7 @@ sub ArticleGet {
         $Data{Body} = $Row[7];
         $Data{Age} = $Self->{TimeObject}->SystemTime() - $Row[8];
         $Ticket{CreateTimeUnix} = $Row[8];
+        $Ticket{Created} = $Self->{TimeObject}->SystemTime2TimeStamp(SystemTime => $Row[8]);
 #        $Ticket{Age} = $Data{Age},
         $Data{PriorityID} = $Row[18];
         $Ticket{PriorityID} = $Row[18];
@@ -1284,34 +1320,57 @@ sub ArticleGet {
         $Data{TicketFreeText16} = $Row[63];
         $Data{TicketFreeTime1} = $Row[67];
         $Data{TicketFreeTime2} = $Row[68];
-
+        $Data{TicketFreeTime3} = $Row[69];
+        $Data{TicketFreeTime4} = $Row[70];
+        $Data{TicketFreeTime5} = $Row[71];
+        $Data{TicketFreeTime6} = $Row[72];
         $Data{IncomingTime} = $Row[30];
         $Data{RealTillTimeNotUsed} = $Row[17];
         $Ticket{LockID} = $Row[64];
+        $Data{TypeID} = $Row[73];
+        $Ticket{TypeID} = $Row[73];
+        $Data{ServiceID} = $Row[74];
+        $Ticket{ServiceID} = $Row[74];
+        $Data{SLAID} = $Row[75];
+        $Ticket{SLAID} = $Row[75];
         # strip not wanted stuff
         foreach (qw(From To Cc Subject)) {
             $Data{$_} =~ s/\n|\r//g if ($Data{$_});
         }
-        push (@Content, {%Data, %Ticket});
+        push (@Content, {%Ticket, %Data});
     }
+    # get type
+    $Ticket{Type} = $Self->{TypeObject}->TypeLookup(TypeID => $Ticket{TypeID} || 1);
     # get owner
     $Ticket{Owner} = $Self->{UserObject}->UserLookup(UserID => $Ticket{OwnerID});
     # get responsible
     $Ticket{Responsible} = $Self->{UserObject}->UserLookup(UserID => $Ticket{ResponsibleID} || 1);
     # get priority
-    $Ticket{Priority} = $Self->PriorityLookup(ID => $Ticket{PriorityID});
+    $Ticket{Priority} = $Self->{PriorityObject}->PriorityLookup(PriorityID => $Ticket{PriorityID});
     # get lock
-    $Ticket{Lock} = $Self->{LockObject}->LockLookup(ID => $Ticket{LockID});
+    $Ticket{Lock} = $Self->{LockObject}->LockLookup(LockID => $Ticket{LockID});
+    # get service
+    if ($Ticket{ServiceID}) {
+        $Ticket{Service} = $Self->{ServiceObject}->ServiceLookup(ServiceID => $Ticket{ServiceID});
+    }
+    # get sla
+    if ($Ticket{SLAID}) {
+        $Ticket{SLA} = $Self->{SLAObject}->SLALookup(SLAID => $Ticket{SLAID});
+    }
     # get queue name and other stuff
     my %Queue = $Self->{QueueObject}->QueueGet(ID => $Ticket{QueueID}, Cache => 1);
     # get state info
     my %StateData = $Self->{StateObject}->StateGet(ID => $Ticket{StateID}, Cache => 1);
+    $Ticket{StateType} = $StateData{TypeName};
+    $Ticket{State} = $StateData{Name};
+    # get esclation attributes
+    my %Escalation = $Self->TicketEscalationState(Ticket => \%Ticket, UserID => $Param{UserID} || 1);
+    %Ticket = (%Escalation, %Ticket);
 
     # article stuff
-    my $EscalationTime = 0;
-    my $EscalationTimeLong = 0;
-    my $EscalationDate = 0;
     foreach my $Part (@Content) {
+        # get type
+        $Part->{Type} = $Ticket{Type};
         # get owner
         $Part->{Owner} = $Ticket{Owner};
         # get responsible
@@ -1329,6 +1388,8 @@ sub ArticleGet {
         $Part->{LockID} = $Ticket{LockID};
         $Part->{Lock} = $Ticket{Lock};
         $Part->{Queue} = $Queue{Name};
+        $Part->{Service} = $Ticket{Service} || '';
+        $Part->{SLA} = $Ticket{SLA} || '';
         if (!$Part->{RealTillTimeNotUsed} || $StateData{TypeName} !~ /^pending/i) {
             $Part->{UntilTime} = 0;
         }
@@ -1339,25 +1400,10 @@ sub ArticleGet {
         $Part->{State} = $StateData{Name};
     }
     # get escalation time
-    if ($Queue{EscalationTime} && $Ticket{EscalationStartTime} && ($Param{TicketID}||$Param{TicketOverTime}) && $StateData{TypeName} !~ /^close/i) {
-        my $CountedTime = $Self->{TimeObject}->WorkingTime(
-            StartTime => $Ticket{EscalationStartTime},
-            StopTime => $Self->{TimeObject}->SystemTime(),
-            Calendar => $Queue{Calendar},
-        );
-        $EscalationTime = ($Queue{EscalationTime}*60) - $CountedTime;
-        my $CountedTime1 = $Self->{TimeObject}->DestinationTime(
-            StartTime => $Ticket{EscalationStartTime},
-            Time => $Queue{EscalationTime}*60,
-            Calendar => $Queue{Calendar},
-        );
-        $EscalationDate = $Self->{TimeObject}->SystemTime2TimeStamp(SystemTime => $CountedTime1);
-        $EscalationTimeLong = $CountedTime1 - $Self->{TimeObject}->SystemTime();
-    }
     foreach my $Part (@Content) {
-        $Part->{TicketOverTime} = $EscalationTime;
-        $Part->{TicketOverTimeLong} = $EscalationTimeLong;
-        $Part->{TicketOverDate} = $EscalationDate;
+        foreach (keys %Escalation) {
+            $Part->{$_} = $Escalation{$_};
+        }
     }
     if ($Param{ArticleID}) {
         return %{$Content[0]};
@@ -1373,13 +1419,13 @@ update a article item
 
 Note: Key "Body", "Subject", "From", "To" and "Cc" is implemented.
 
-  $TicketObject->ArticleUpdate(
-      ArticleID => 123,
-      Key => 'Body',
-      Value => 'New Body',
-      UserID => 123,
-      TicketID => 123,
-  );
+    $TicketObject->ArticleUpdate(
+        ArticleID => 123,
+        Key => 'Body',
+        Value => 'New Body',
+        UserID => 123,
+        TicketID => 123,
+    );
 
 =cut
 
@@ -1435,53 +1481,51 @@ sub ArticleUpdate {
 
 send article via email and create article with attachments
 
-  my $ArticleID = $TicketObject->ArticleSend(
-      TicketID => 123,
-      ArticleType => 'note-internal' # email-external|email-internal|phone|fax|...
-      SenderType => 'agent', # agent|system|customer
-      From => 'Some Agent <email@example.com>', # not required but useful
-      To => 'Some Customer A <customer-a@example.com>', # not required but useful
-      Cc => 'Some Customer B <customer-b@example.com>', # not required but useful
-      ReplyTo => 'Some Customer B <customer-b@example.com>', # not required
-      Subject => 'some short description', # required
-      Body => 'the message text', # required
-      MessageID => '<asdasdasd.123@example.com>', # not required but useful
-      Charset => 'ISO-8859-15'
-      Type => 'text/plain',
-      Loop => 0, # 1|0 used for bulk emails
-      Attachment => [
-        {
-          Content => $Content,
-          ContentType => $ContentType,
-          Filename => 'lala.txt',
+    my $ArticleID = $TicketObject->ArticleSend(
+        TicketID => 123,
+        ArticleType => 'note-internal' # email-external|email-internal|phone|fax|...
+        SenderType => 'agent', # agent|system|customer
+        From => 'Some Agent <email@example.com>', # not required but useful
+        To => 'Some Customer A <customer-a@example.com>', # not required but useful
+        Cc => 'Some Customer B <customer-b@example.com>', # not required but useful
+        ReplyTo => 'Some Customer B <customer-b@example.com>', # not required
+        Subject => 'some short description', # required
+        Body => 'the message text', # required
+        MessageID => '<asdasdasd.123@example.com>', # not required but useful
+        Charset => 'ISO-8859-15'
+        Type => 'text/plain',
+        Loop => 0, # 1|0 used for bulk emails
+        Attachment => [
+            {
+                Content => $Content,
+                ContentType => $ContentType,
+                Filename => 'lala.txt',
+            },
+            {
+                Content => $Content,
+                ContentType => $ContentType,
+                Filename => 'lala1.txt',
+            },
+        ],
+        Sign => {
+            Type => 'PGP',
+            SubType => 'Inline|Detached',
+            Key => '81877F5E',
+            Type => 'SMIME',
+            Key => '3b630c80',
         },
-        {
-          Content => $Content,
-          ContentType => $ContentType,
-          Filename => 'lala1.txt',
+        Crypt => {
+            Type => 'PGP',
+            SubType => 'Inline|Detached',
+            Key => '81877F5E',
+            Type => 'SMIME',
+            Key => '3b630c80',
         },
-      ],
-      Sign => {
-          Type => 'PGP',
-          SubType => 'Inline|Detached',
-          Key => '81877F5E',
-
-          Type => 'SMIME',
-          Key => '3b630c80',
-      },
-      Crypt => {
-          Type => 'PGP',
-          SubType => 'Inline|Detached',
-          Key => '81877F5E',
-
-          Type => 'SMIME',
-          Key => '3b630c80',
-      },
-      HistoryType => 'OwnerUpdate',  # Move|AddNote|PriorityUpdate|WebRequestCustomer|...
-      HistoryComment => 'Some free text!',
-      NoAgentNotify => 0,            # if you don't want to send agent notifications
-      UserID => 123,
-  );
+        HistoryType => 'OwnerUpdate',  # Move|AddNote|PriorityUpdate|WebRequestCustomer|...
+        HistoryComment => 'Some free text!',
+        NoAgentNotify => 0,            # if you don't want to send agent notifications
+        UserID => 123,
+    );
 
 =cut
 
@@ -1512,7 +1556,6 @@ sub ArticleSend {
     # clean up
     $Param{Body} =~ s/(\r\n|\n\r)/\n/g;
     $Param{Body} =~ s/\r/\n/g;
-
     # create article
     my $MessageID = "<$Time.$Random.$Param{TicketID}.$Param{UserID}\@$Self->{FQDN}>";
     if ($Param{ArticleID} = $Self->ArticleCreate(
@@ -1520,7 +1563,7 @@ sub ArticleSend {
         ContentType => "$Param{Type}, charset=$Param{Charset}",
         MessageID => $MessageID,
     )) {
-      ####
+        # no action
     }
     else {
         return;
@@ -1591,8 +1634,8 @@ sub ArticleSend {
         return $Param{ArticleID};
     }
     else {
-         # error
-         return;
+        # error
+        return;
     }
 }
 
@@ -1600,14 +1643,14 @@ sub ArticleSend {
 
 bounce an article
 
-  $TicketObject->ArticleBounce(
-      From => '',
-      To => '',
-      Email => $EmailAsString,
-      TicketID => 123,
-      ArticleID => 123,
-      UserID => 123,
-  );
+    $TicketObject->ArticleBounce(
+        From => '',
+        To => '',
+        Email => $EmailAsString,
+        TicketID => 123,
+        ArticleID => 123,
+        UserID => 123,
+    );
 
 =cut
 
@@ -1656,15 +1699,15 @@ sub ArticleBounce {
 
 send an agent notification via email
 
-  $TicketObject->SendAgentNotification(
-      TicketID => 123,
-      CustomerMessageParams => {
-         SomeParams => 'For the message!',
-      },
-      Type => 'Move', # notification types, see database
-      UserData => { $UserObject->GetUserData(UserID => 3123)}
-      UserID => 123,
-  );
+    $TicketObject->SendAgentNotification(
+        TicketID => 123,
+        CustomerMessageParams => {
+            SomeParams => 'For the message!',
+        },
+        Type => 'Move', # notification types, see database
+        UserData => { $UserObject->GetUserData(UserID => 3123)}
+        UserID => 123,
+    );
 
 =cut
 
@@ -1769,7 +1812,6 @@ sub SendAgentNotification {
             $Notification{Subject} =~ s/<OTRS_$_>/$User{$_}/gi;
         }
     }
-
     # ticket data
     my %Ticket = $Self->TicketGet(TicketID => $Param{TicketID});
     foreach (keys %Ticket) {
@@ -1892,14 +1934,14 @@ sub SendAgentNotification {
 
 send a customer notification via email
 
-  my $ArticleID = $TicketObject->SendCustomerNotification(
-      Type => 'Move', # notification types, see database
-      CustomerMessageParams => {
-         SomeParams => 'For the message!',
-      },
-      TicketID => 123,
-      UserID => 123,
-  );
+    my $ArticleID = $TicketObject->SendCustomerNotification(
+        Type => 'Move', # notification types, see database
+        CustomerMessageParams => {
+            SomeParams => 'For the message!',
+        },
+        TicketID => 123,
+        UserID => 123,
+    );
 
 =cut
 
@@ -2150,18 +2192,18 @@ sub SendCustomerNotification {
 
 send an auto response to a customer via email
 
-  my $ArticleID = $TicketObject->SendAutoResponse(
-      TicketID => 123,
-      TicketNumber => '123123123',
-      Text => 'Quote Message',
-      Realname => 'Support Team',
-      Address => 'support@example.com',
-      HistoryType => 'SendAutoReply', # SendAutoReply|SendAutoReject|SendAutoFollowUp|...
-      CustomerMessageParams => {
-         SomeParams => 'For the message!',
-      },
-      UserID => 123,
-  );
+    my $ArticleID = $TicketObject->SendAutoResponse(
+        TicketID => 123,
+        TicketNumber => '123123123',
+        Text => 'Quote Message',
+        Realname => 'Support Team',
+        Address => 'support@example.com',
+        HistoryType => 'SendAutoReply', # SendAutoReply|SendAutoReject|SendAutoFollowUp|...
+        CustomerMessageParams => {
+            SomeParams => 'For the message!',
+        },
+        UserID => 123,
+    );
 
 =cut
 
@@ -2203,7 +2245,7 @@ sub SendAutoResponse {
         $Self->{LogObject}->Log(
             Priority => 'notice',
             Message => "Sent not auto response to '$GetParam{From}' because config".
-             " option SendNoAutoResponseRegExp (/$NoAutoRegExp/i) is matching!",
+                " option SendNoAutoResponseRegExp (/$NoAutoRegExp/i) is matching!",
         );
         return 1;
     }
@@ -2393,7 +2435,7 @@ sub SendAutoResponse {
     $Self->{LogObject}->Log(
         Priority => 'notice',
         Message => "Sent auto response ($Param{HistoryType}) for Ticket [$Article{TicketNumber}]".
-         " (TicketID=$Param{TicketID}, ArticleID=$ArticleID) to '$ToAll'."
+            " (TicketID=$Param{TicketID}, ArticleID=$ArticleID) to '$ToAll'."
     );
 
     # ticket event
@@ -2410,11 +2452,11 @@ sub SendAutoResponse {
 
 set article flags
 
-  $TicketObject->ArticleFlagSet(
-      ArticleID => 123,
-      Flag => 'seen',
-      UserID => 123,
-  );
+    $TicketObject->ArticleFlagSet(
+        ArticleID => 123,
+        Flag => 'seen',
+        UserID => 123,
+    );
 
 =cut
 
@@ -2463,10 +2505,10 @@ sub ArticleFlagSet {
 
 get article flags
 
-  my %Article = $TicketObject->ArticleFlagGet(
-      ArticleID => 123,
-      UserID => 123,
-  );
+    my %Article = $TicketObject->ArticleFlagGet(
+        ArticleID => 123,
+        UserID => 123,
+    );
 
 =cut
 
@@ -2488,7 +2530,7 @@ sub ArticleFlagGet {
     # sql query
     $Self->{DBObject}->Prepare(
         SQL => "SELECT article_flag FROM article_flag WHERE ".
-          " article_id = $Param{ArticleID} AND create_by = $Param{UserID}",
+            " article_id = $Param{ArticleID} AND create_by = $Param{UserID}",
         Limit => 1000,
     );
     while (my @Row = $Self->{DBObject}->FetchrowArray()) {
@@ -2501,9 +2543,9 @@ sub ArticleFlagGet {
 
 returns the accounted time of a article.
 
-  my $AccountedTime = $TicketObject->ArticleAccountedTimeGet(
-    ArticleID => $ArticleID,
-  );
+    my $AccountedTime = $TicketObject->ArticleAccountedTimeGet(
+        ArticleID => $ArticleID,
+    );
 
 =cut
 
@@ -2538,9 +2580,9 @@ sub ArticleAccountedTimeGet {
 
 delete accounted time of article
 
-  $TicketObject->ArticleAccountedTimeDelete(
-    ArticleID => $ArticleID,
-  );
+    $TicketObject->ArticleAccountedTimeDelete(
+        ArticleID => $ArticleID,
+    );
 
 =cut
 
@@ -2563,85 +2605,85 @@ sub ArticleAccountedTimeDelete {
     return $Self->{DBObject}->Do(SQL => $SQL);
 }
 
+1;
+
 # the following is the pod for Kernel/System/Ticket/ArticleStorage*.pm
 
 =item ArticleDelete()
 
 delete all article, attachments and plain message of a ticket
 
-  $TicketObject->ArticleDelete(
-      TicketID => 123,
-      UserID => 123,
-  );
+    $TicketObject->ArticleDelete(
+        TicketID => 123,
+        UserID => 123,
+    );
 
 =item ArticleDeletePlain()
 
 delete a artile plain message
 
-  $TicketObject->ArticleDeletePlain(
-      ArticleID => 123,
-      UserID => 123,
-  );
+    $TicketObject->ArticleDeletePlain(
+        ArticleID => 123,
+        UserID => 123,
+    );
 
 =item ArticleDeleteAttachment()
 
 delete all attachments of an article
 
-  $TicketObject->ArticleDeleteAttachment(
-      ArticleID => 123,
-      UserID => 123,
-  );
+    $TicketObject->ArticleDeleteAttachment(
+        ArticleID => 123,
+        UserID => 123,
+    );
 
 =item ArticleWritePlain()
 
 write an plain email to storage
 
-  $TicketObject->ArticleWritePlain(
-      ArticleID => 123,
-      Email => $EmailAsString,
-      UserID => 123,
-  );
+    $TicketObject->ArticleWritePlain(
+        ArticleID => 123,
+        Email => $EmailAsString,
+        UserID => 123,
+    );
 
 =item ArticlePlain()
 
 get plain message/email
 
-  my $PlainMessage = $TicketObject->ArticlePlain(
-      ArticleID => 123,
-      UserID => 123,
-  );
+    my $PlainMessage = $TicketObject->ArticlePlain(
+        ArticleID => 123,
+        UserID => 123,
+    );
 
 =item ArticleWriteAttachment()
 
 write an article attachemnt to storage
 
-  $TicketObject->ArticleWriteAttachment(
-      Content => $ContentAsString,
-      ContentType => 'text/html; charset="iso-8859-15"',
-      Filename => 'lala.html',
-      ArticleID => 123,
-      UserID => 123,
-  );
+    $TicketObject->ArticleWriteAttachment(
+        Content => $ContentAsString,
+        ContentType => 'text/html; charset="iso-8859-15"',
+        Filename => 'lala.html',
+        ArticleID => 123,
+        UserID => 123,
+    );
 
 =item ArticleAttachmentIndex()
 
 get article attachment index as hash (ID => hashref (Filename, Filesize))
 
-  my %Index = $TicketObject->ArticleAttachmentIndex(
-      ArticleID => 123,
-      UserID => 123,
-  );
+    my %Index = $TicketObject->ArticleAttachmentIndex(
+        ArticleID => 123,
+        UserID => 123,
+    );
 
 =item ArticleAttachment()
 
 get article attachment (Content, ContentType, Filename)
 
-  my %Attachment = $TicketObject->ArticleAttachment(
-      ArticleID => 123,
-      FileID => 1,
-      UserID => 123,
-  );
+    my %Attachment = $TicketObject->ArticleAttachment(
+        ArticleID => 123,
+        FileID => 1,
+        UserID => 123,
+    );
 
 =cut
-
-1;
